extend("fiskheroes:hero_basic");
loadTextures({
    "layer1": "smp:infinity_ultron_layer1",
    "layer2": "smp:infinity_ultron_layer2",
    "cape": "fiskheroes:vision_cape",
    "mind": "smp:infinity_ultron_mind_stone",
    "space": "smp:infinity_ultron_space_stone",
    "power": "smp:infinity_ultron_power_stone",
    "lance": "smp:infinity_ultron_lance"
});

var utils = implement("fiskheroes:external/utils");
var capes = implement("fiskheroes:external/capes");

var cape;
var mind;
var space;

function initEffects(renderer) {
    var physics = renderer.createResource("CAPE_PHYSICS", null);
    physics.weight = 0.9;
    physics.maxFlare = 0.5;
    cape = capes.createDefault(renderer, 24, "fiskheroes:cape_default.mesh.json", physics);
    cape.effect.texture.set("cape");
    cape.effect.opacity = 0.85;

    mind = renderer.createEffect("fiskheroes:overlay");
    mind.texture.set(null, "mind");

    space = renderer.createEffect("fiskheroes:overlay");
    space.texture.set(null, "space");

    power = renderer.createEffect("fiskheroes:overlay");
    power.texture.set(null, "power");

    utils.setOpacityWithData(renderer, 0.5, 1.0, "fiskheroes:intangibility_timer");
    utils.bindBeam(renderer, "fiskheroes:charged_beam", "fiskheroes:charged_beam", "head", 0xFFFF6D, [
        { "firstPerson": [0.0, -6.15, 0.0], "offset": [0.0, -6.15, -4.0], "size": [1.0, 0.5] }
    ]).setParticles(renderer.createResource("PARTICLE_EMITTER", "fiskheroes:impact_charged_beam"));

    utils.bindBeam(renderer, "fiskheroes:energy_projection", "smp:superman_heat_vision", "body", 0xAD09C6, [
        { "firstPerson": [0.0, 6.0, 0.0], "offset": [1.5, 2.5, -2.0], "size": [1.0, 1.0] }
    ]).setParticles(renderer.createResource("PARTICLE_EMITTER", "fiskheroes:impact_charged_beam"));

    utils.bindCloud(renderer, "fiskheroes:telekinesis", "smp:space_stone");

    var forcefield = renderer.bindProperty("fiskheroes:forcefield");
    forcefield.setShape(36, 18).setOffset(0.0, 6.0, 0.0).setScale(1.5);
    forcefield.setCondition(entity => {
        forcefield.opacity = (entity.getData("fiskheroes:mask_open") ? entity.getInterpolatedData("fiskheroes:mask_open_timer2") : entity.getInterpolatedData("fiskheroes:shield_blocking_timer")) * 0.5;
        forcefield.color.set(entity.getData("fiskheroes:mask_open") ? 0xFF0000 : 0x3908FF);
        return true;
    });

    lance = renderer.createEffect("fiskheroes:model");
    lance.setModel(utils.createModel(renderer, "smp:infinity_ultron_lance", "lance"));
    lance.anchor.set("rightArm");
    lance.setRotation(90, 0, 0);
    lance.setOffset(1, 8, -19.5)
}

function initAnimations(renderer) {
    parent.initAnimations(renderer);
    renderer.removeCustomAnimation("basic.BLOCKING");
    renderer.removeCustomAnimation("basic.ENERGY_PROJ");

    utils.addFlightAnimation(renderer, "vision.FLIGHT", "fiskheroes:flight/default.anim.json");
    utils.addHoverAnimation(renderer, "vision.HOVER", "fiskheroes:flight/idle/neutral");
}

function render(entity, renderLayer, isFirstPersonArm) {
    if (renderLayer == "CHESTPLATE" && entity.getData("fiskheroes:blade")) {
        lance.render();
    }
    if (!isFirstPersonArm) {
        if (renderLayer == "HELMET") {
            mind.opacity = entity.getInterpolatedData("fiskheroes:beam_charge");
            mind.render();
        }
        else if (renderLayer == "CHESTPLATE") {
            cape.render(entity);

            space.opacity = entity.getData("fiskheroes:telekinesis") ? 1 : entity.getInterpolatedData("fiskheroes:shield_blocking_timer")
            space.render();

            power.opacity = entity.getInterpolatedData("fiskheroes:energy_projection_timer")
            power.render();
        }
    }
}
